==> azorius-0.3.2.tgz: azorius-0.3.2/notifs.go <==
//
// Copyright (c) 2023 Ted Unangst <tedu@tedunangst.com>
//
// Permission to use, copy, modify, and distribute this software for any
// purpose with or without fee is hereby granted, provided that the above
// copyright notice and this permission notice appear in all copies.
//
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
// WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
// ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
// ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
// OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

package main

import (
	"database/sql"
	"fmt"
	"sync"
	"time"
)

var userNotifs = make(map[int64]int64)
var mtxNotifs sync.Mutex

func numNotifs(userid int64) int64 {
	mtxNotifs.Lock()
	defer mtxNotifs.Unlock()
	return userNotifs[userid]
}

func incNotifs(userid int64) {
	mtxNotifs.Lock()
	defer mtxNotifs.Unlock()
	userNotifs[userid] += 1
}

func clearNotifs(userid int64) {
	mtxNotifs.Lock()
	defer mtxNotifs.Unlock()
	if userNotifs[userid] > 0 {
		err := dbClearNotifs(userid)
		if err != nil {
			elog.Printf("error clearing notifs: %s", err)
		}
		userNotifs[userid] = 0
	}
}

func initNotifs() {
	db := opendatabase()
	rows, err := db.Query("select userid, count(notifid) from notifs where flags & 1 = 1 group by userid")
	if err != nil {
		elog.Printf("error getting notif counts: %s", err)
		return
	}
	defer rows.Close()
	for rows.Next() {
		var userid, count int64
		err = rows.Scan(&userid, &count)
		if err != nil {
			elog.Printf("error scanning notif count: %s", err)
			continue
		}
		userNotifs[userid] = count
	}
}

func notifReset(userid int64, msg, url string) {
	del := dbDeleteNewNotif(userid, msg, url)
	err := dbSaveNotif(userid, msg, url)
	if err != nil {
		elog.Printf("error saving notif: %s", err)
	}
	if !del {
		incNotifs(userid)
	}
}

func notifSave(userid int64, msg, url string) {
	err := dbSaveNotif(userid, msg, url)
	if err != nil {
		elog.Printf("error saving notif: %s", err)
	}
	incNotifs(userid)
}

func notifAdmins(msg, url string) {
	for _, admin := range adminUsers {
		notifSave(admin, msg, url)
	}
}

func notifNewReport(report *Report) {
	notifAdmins(fmt.Sprintf("new report on %s", report.What), report.Link)
}

func notifNewCommentOnPost(post *Post, comment *Comment) {
	subs := post.Subs()
	for _, userid := range subs {
		if userid == comment.PersonID {
			continue
		}
		notifSave(userid, fmt.Sprintf("new comment from %s on %s", comment.PersonName, post.Title), post.Rel())
	}
	if post.IsRemote() {
		return
	}
	if comment.PersonID == post.PersonID {
		return
	}
	user := finduser(post.PersonName)
	notifSave(user.ID, fmt.Sprintf("new comment from %s on %s", comment.PersonName, post.Title), post.Rel())
}

func notifNewCommentReply(post *Post, parent *Comment, comment *Comment) {
	if parent.IsRemote() {
		return
	}
	if comment.PersonID == parent.PersonID {
		return
	}
	user := finduser(parent.PersonName)
	notifSave(user.ID, fmt.Sprintf("new reply from %s on %s", comment.PersonName, post.Title), post.Rel())
}

func notifNewComment(post *Post, parent *Comment, comment *Comment) {
	if parent != nil {
		notifNewCommentReply(post, parent, comment)
	}
	notifNewCommentOnPost(post, comment)
}

func notifNewPost(group *Group, post *Post) {
	if group.IsRemote() {
		return
	}
	for _, mod := range group.Mods {
		if mod == post.PersonName {
			continue
		}
		user := finduser(mod)
		notifSave(user.ID, fmt.Sprintf("new post from %s in %s: %s", post.PersonName, group.Name, post.Title), post.Rel())
	}
}

func notifNewChat(userid int64, group *Group, chat *Chat) {
	for _, sub := range group.Subs() {
		if sub == userid {
			continue
		}
		notifReset(sub, fmt.Sprintf("new message in the %s chat", group.Name), group.Rel()+"/chat")
	}
}

func notifNewGroup(group *Group) {
	notifAdmins(fmt.Sprintf("new group: %s", group.Name), group.Rel())
}

func notifNewUser(user *User) {
	notifAdmins(fmt.Sprintf("new user: %s", user.Name), user.Person.Rel())
}

func dbSaveNotif(userid int64, msg, url string) error {
	dt := time.Now().UTC().Format(dbtimeformat)
	flags := 1
	_, err := stmtSaveNotif.Exec(userid, dt, flags, msg, url)
	return err
}

func dbDeleteNewNotif(userid int64, msg, url string) bool {
	res, err := stmtDeleteNewNotif.Exec(userid, msg, url)
	if err != nil {
		elog.Printf("error deleting notif: %s", err)
		return false
	}
	num, _ := res.RowsAffected()
	return num > 0
}

func dbClearNotifs(userid int64) error {
	_, err := stmtClearNotifs.Exec(userid)
	return err
}

func scannotif(row RowLike) *Notif {
	notif := new(Notif)
	var dt string
	err := row.Scan(&notif.ID, &notif.UserID, &dt, &notif.Flags, &notif.Msg, &notif.Url)
	if err == sql.ErrNoRows {
		return nil
	}
	if err == nil {
		notif.Date, _ = time.Parse(dbtimeformat, dt)
	}
	if err != nil {
		elog.Printf("error scanning notif: %s", err)
		return nil
	}
	return notif
}

func getnotifs(userid int64) []*Notif {
	var notifs []*Notif
	rows, err := stmtGetNotifs.Query(userid)
	if err != nil {
		elog.Printf("error getting notifs: %s", err)
		return notifs
	}
	defer rows.Close()
	for rows.Next() {
		notif := scannotif(rows)
		if notif == nil {
			continue
		}
		notifs = append(notifs, notif)
	}
	return notifs
}

==> azorius-0.3.3.tgz: azorius-0.3.3/notifs.go <==
//
// Copyright (c) 2023 Ted Unangst <tedu@tedunangst.com>
//
// Permission to use, copy, modify, and distribute this software for any
// purpose with or without fee is hereby granted, provided that the above
// copyright notice and this permission notice appear in all copies.
//
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
// WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
// ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
// ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
// OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

package main

import (
	"database/sql"
	"fmt"
	"sync"
	"time"
)

var userNotifs = make(map[int64]int64)
var mtxNotifs sync.Mutex

func numNotifs(userid int64) int64 {
	mtxNotifs.Lock()
	defer mtxNotifs.Unlock()
	return userNotifs[userid]
}

func incNotifs(userid int64) {
	mtxNotifs.Lock()
	defer mtxNotifs.Unlock()
	userNotifs[userid] += 1
}

func clearNotifs(userid int64) {
	mtxNotifs.Lock()
	defer mtxNotifs.Unlock()
	if userNotifs[userid] > 0 {
		err := dbClearNotifs(userid)
		if err != nil {
			elog.Printf("error clearing notifs: %s", err)
		}
		userNotifs[userid] = 0
	}
}

func initNotifs() {
	db := opendatabase()
	rows, err := db.Query("select userid, count(notifid) from notifs where flags & 1 = 1 group by userid")
	if err != nil {
		elog.Printf("error getting notif counts: %s", err)
		return
	}
	defer rows.Close()
	for rows.Next() {
		var userid, count int64
		err = rows.Scan(&userid, &count)
		if err != nil {
			elog.Printf("error scanning notif count: %s", err)
			continue
		}
		userNotifs[userid] = count
	}
}

func notifReset(userid int64, msg, url string) {
	del := dbDeleteNewNotif(userid, msg, url)
	err := dbSaveNotif(userid, msg, url)
	if err != nil {
		elog.Printf("error saving notif: %s", err)
	}
	if !del {
		incNotifs(userid)
	}
}

func notifSave(userid int64, msg, url string) {
	err := dbSaveNotif(userid, msg, url)
	if err != nil {
		elog.Printf("error saving notif: %s", err)
	}
	incNotifs(userid)
}

func notifAdmins(msg, url string) {
	for _, admin := range adminUsers {
		notifSave(admin, msg, url)
	}
}

func notifNewReport(report *Report) {
	notifAdmins(fmt.Sprintf("new report on %s", report.What), report.Link)
}

func notifNewCommentOnPost(post *Post, comment *Comment) {
	link := fmt.Sprintf("%s#c%s", post.Rel(), comment.Xid)
	subs := post.Subs()
	for _, userid := range subs {
		if userid == comment.PersonID {
			continue
		}
		notifSave(userid, fmt.Sprintf("new comment from %s on %s", comment.PersonName, post.Title), link)
	}
	if post.IsRemote() {
		return
	}
	if comment.PersonID == post.PersonID {
		return
	}
	user := finduser(post.PersonName)
	notifSave(user.ID, fmt.Sprintf("new comment from %s on %s", comment.PersonName, post.Title), link)
}

func notifNewCommentReply(post *Post, parent *Comment, comment *Comment) {
	link := fmt.Sprintf("%s#c%s", post.Rel(), comment.Xid)
	if parent.IsRemote() {
		return
	}
	if comment.PersonID == parent.PersonID {
		return
	}
	user := finduser(parent.PersonName)
	notifSave(user.ID, fmt.Sprintf("new reply from %s on %s", comment.PersonName, post.Title), link)
}

func notifNewComment(post *Post, parent *Comment, comment *Comment) {
	if parent != nil {
		notifNewCommentReply(post, parent, comment)
	}
	notifNewCommentOnPost(post, comment)
}

func notifNewPost(group *Group, post *Post) {
	if group.IsRemote() {
		return
	}
	for _, mod := range group.Mods {
		if mod == post.PersonName {
			continue
		}
		user := finduser(mod)
		notifSave(user.ID, fmt.Sprintf("new post from %s in %s: %s", post.PersonName, group.Name, post.Title), post.Rel())
	}
}

func notifNewChat(userid int64, group *Group, chat *Chat) {
	for _, sub := range group.Subs() {
		if sub == userid {
			continue
		}
		notifReset(sub, fmt.Sprintf("new message in the %s chat", group.Name), group.Rel()+"/chat")
	}
}

func notifNewGroup(group *Group) {
	notifAdmins(fmt.Sprintf("new group: %s", group.Name), group.Rel())
}

func notifNewUser(user *User) {
	notifAdmins(fmt.Sprintf("new user: %s", user.Name), user.Person.Rel())
}

func dbSaveNotif(userid int64, msg, url string) error {
	dt := time.Now().UTC().Format(dbtimeformat)
	flags := 1
	_, err := stmtSaveNotif.Exec(userid, dt, flags, msg, url)
	return err
}

func dbDeleteNewNotif(userid int64, msg, url string) bool {
	res, err := stmtDeleteNewNotif.Exec(userid, msg, url)
	if err != nil {
		elog.Printf("error deleting notif: %s", err)
		return false
	}
	num, _ := res.RowsAffected()
	return num > 0
}

func dbClearNotifs(userid int64) error {
	_, err := stmtClearNotifs.Exec(userid)
	return err
}

func scannotif(row RowLike) *Notif {
	notif := new(Notif)
	var dt string
	err := row.Scan(&notif.ID, &notif.UserID, &dt, &notif.Flags, &notif.Msg, &notif.Url)
	if err == sql.ErrNoRows {
		return nil
	}
	if err == nil {
		notif.Date, _ = time.Parse(dbtimeformat, dt)
	}
	if err != nil {
		elog.Printf("error scanning notif: %s", err)
		return nil
	}
	return notif
}

func getnotifs(userid int64) []*Notif {
	var notifs []*Notif
	rows, err := stmtGetNotifs.Query(userid)
	if err != nil {
		elog.Printf("error getting notifs: %s", err)
		return notifs
	}
	defer rows.Close()
	for rows.Next() {
		notif := scannotif(rows)
		if notif == nil {
			continue
		}
		notifs = append(notifs, notif)
	}
	return notifs
}
